
/******************************************************************************
* MODULE     : interval_mul.cpp
* DESCRIPTION: Multiplication of intervals
* COPYRIGHT  : (C) 2003  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "interval_impl.hpp"
namespace mml {

interval
operator * (const interval& x, const interval& y) {
  if (is_error (x)) return x;
  if (is_error (y)) return y;

  if (has_low_prec (x) || has_low_prec (y)) {
    interval X= reduce (x);
    interval Y= reduce (y);
    interval_rep* r= raw_interval ();
    if (mpfr_sgn (X->min) >= 0) {
      if (mpfr_sgn (Y->min) >= 0) mpfr_mul (r->min, X->min, Y->min, GMP_RNDD);
      else mpfr_mul (r->min, X->max, Y->min, GMP_RNDD);
      if (mpfr_sgn (Y->max) >= 0) mpfr_mul (r->max, X->max, Y->max, GMP_RNDU);
      else mpfr_mul (r->max, X->min, Y->max, GMP_RNDU);
    }
    else if (mpfr_sgn (X->max) <= 0) {
      if (mpfr_sgn (Y->max) >= 0) mpfr_mul (r->min, X->min, Y->max, GMP_RNDD);
      else mpfr_mul (r->min, X->max, Y->max, GMP_RNDD);
      if (mpfr_sgn (Y->min) >= 0) mpfr_mul (r->max, X->max, Y->min, GMP_RNDU);
      else mpfr_mul (r->max, X->min, Y->min, GMP_RNDU);
    }
    else if (mpfr_sgn (Y->min) >= 0) {
      mpfr_mul (r->min, X->min, Y->max, GMP_RNDD);
      mpfr_mul (r->max, X->max, Y->max, GMP_RNDU);
    }
    else if (mpfr_sgn (Y->max) <= 0) {
      mpfr_mul (r->min, X->max, Y->min, GMP_RNDD);
      mpfr_mul (r->max, X->min, Y->min, GMP_RNDU);
    }
    else {
      lp_dyadic min_s, max_s;
      mpfr_mul (r->min, X->min, Y->max, GMP_RNDD);
      mpfr_mul (*min_s, X->max, Y->min, GMP_RNDD);
      if (mpfr_cmp (*min_s, r->min) < 0)
	mpfr_set (r->min, *min_s, GMP_RNDD);
      mpfr_mul (r->max, X->min, Y->min, GMP_RNDU);
      mpfr_mul (*max_s, X->max, Y->max, GMP_RNDU);
      if (mpfr_cmp (*max_s, r->max) > 0)
	mpfr_set (r->min, *min_s, GMP_RNDU);
    }
    return r;
  }

  else {
    interval_rep* r= raw_interval (min (prec (x), prec (y)) + 1);
    lp_dyadic aux;
    mpfr_mul (r->mid, x->mid, y->mid, GMP_RNDN);
    CHECK_UNDERFLOW (r);
    CHECK_OVERFLOW (r);
    mpfr_set_ui_2si (r->rad, 1, mpfr_get_step (r->mid), GMP_RNDU);
    mpfr_abs (*aux, x->mid, GMP_RNDU);
    mpfr_mul (*aux, *aux, y->rad, GMP_RNDU);
    mpfr_add (r->rad, r->rad, *aux, GMP_RNDU);
    mpfr_abs (*aux, y->mid, GMP_RNDU);
    mpfr_mul (*aux, *aux, x->rad, GMP_RNDU);
    mpfr_add (r->rad, r->rad, *aux, GMP_RNDU);
    mpfr_mul (*aux, x->rad, y->rad, GMP_RNDU);
    mpfr_add (r->rad, r->rad, *aux, GMP_RNDU);
    return r->finish_high ();
  }
}

} // namespace mml
