<TeXmacs|1.0.4.5>

<style|mmxdoc>

<\body>
  <tmdoc-title|Symbolic expressions>

  The aim of the <verbatim|symbolic> class is to provide a uniform interface
  for a wide variety of <em|syntactic> symbolic expression types. The
  symbolic class is really a ``contextual template type''
  <verbatim|symbolic\<less\>C\<gtr\>>, with<nbsp><verbatim|C> of type
  <verbatim|context>. This means that one should think of the class as
  carrying a template parameter<nbsp><verbatim|C>, but this parameter is
  really a global variable which may be changed during computations.

  The <verbatim|context> class determines a ``sufficient simple''
  mathematical theory, so that free expressions in the theory can be
  represented in a canonical and syntactic way. For instance, if the context
  is ``free commutative algebra'', then concrete symbolic objects might be
  polynomials. In general, the context rather specifies properties for
  objects, so that certain variables behave like commutative variables, other
  variables are systematically expanded, and yet other variables do not
  satisfy any relations.

  From the internal point of view, symbolic objects are represented by unique
  pointers. In other words, whenever a symbolic expression is constructed, we
  first test whether this symbolic expression already exists in memory and
  use a pointer copy of the expression in memory if this is the
  case<nbsp>(recall: we might retain the <em|simplest> representation). This
  strategy is possible, because we assumed that terms in the theory from the
  context admit canonical expressions. Expressions are compared
  (<verbatim|==>, <samp|<verbatim|!=>>, <verbatim|\<less\>=>,
  <verbatim|\<less\>>, <verbatim|\<gtr\>=>, <verbatim|\<gtr\>>) and hashed
  using their pointers. In the case of <verbatim|\<less\>=>,
  <verbatim|\<less\>>, <verbatim|\<gtr\>=>, <verbatim|\<gtr\>>, this should
  not be mixed up with possible other significations of these relation (in
  which a new class should be constructed on top of <verbatim|symbolic>).

  Typically, the <verbatim|symbolic> class should be thought of as a union of
  classes of the following type

  <\itemize>
    <item>One class for atomic symbols like <with|mode|math|x>,
    <with|mode|math|y>, <abbr|etc.>

    <item>Atomic classes, like integers, rational numbers, etc.

    <item>One class for syntactic compound objects like <with|mode|math|f(x)>
    or <with|mode|math|f(2)>.

    <item>Additional classes for the efficient representation of special
    types of compound objects, like commutative sums, associative products,
    <abbr|etc.>
  </itemize>

  In addition, the symbolic class might carry out a limited amount of term
  rewriting. This presupposes the existence of pattern matching for the
  symbolic type (<abbr|i.e.> for all classes in the union) and the fact that
  the term rewriting ends up with canonical objects (<abbr|i.e.> if the
  Church-Rosser property is satisfied).

  In general, the <verbatim|symbolic> class is <em|not> meant to encode
  objects in complicated models. Instead, the <verbatim|symbolic> class is
  used to provide a suitable syntactical objects (better than reasoning on
  mere trees, since atomic classes and special types of compound classes are
  treated in a more optimal way) to <em|express> and <em|solve> problems in
  complex theories. Of course, whenever, after additional computations, a
  complex theory may be reformulated such that objects again admit canonical
  representations (<abbr|i.e.> reduced Grbner basis and reduced
  polynomials), then canonical objects in the reformulated theory may again
  be reused to recursively build more complicated symbolic classes.

  For experimental purposes, one may cheat with respect to what has been said
  above, and construct symbolic classes which correspond to models in which
  canonical representations are not beforehand. In that case, useful
  computations may still be carried out, but one has to bear in mind that the
  equality test <verbatim|==> in such models is no longer correct. In this
  case, computations in <verbatim|symbolic> only <em|approximate>
  computations in the model one wishes to represent.

  <tmdoc-copyright|2004|Joris van der Hoeven>

  <tmdoc-license|Permission is granted to copy, distribute and/or modify this
  document under the terms of the <hlink|GNU General Public
  License|http://www.gnu.org/licenses/gpl.txt>. If you don't have this file,
  write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  Boston, MA 02111-1307, USA.>
</body>