
/******************************************************************************
* MODULE     : symbolic_arith.cpp
* DESCRIPTION: Adding symbolic expressions
* COPYRIGHT  : (C) 2004  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "symbolic.hpp"
namespace mml {

symbolic
symbolic_rep::add (const symbolic& s1, const symbolic& s2) {
  if (s1 == SYMB_ZERO) return s2;
  if (s2 == SYMB_ZERO) return s1;
  return apply (SYMB_PLUS, s1, s2);
}

symbolic
symbolic_rep::neg (const symbolic& s1) {
  return apply (SYMB_MINUS, s1);
}

symbolic
symbolic_rep::sub (const symbolic& s1, const symbolic& s2) {
  if (s1 == SYMB_ZERO) return s2->neg (s2);
  if (s2 == SYMB_ZERO) return s1;
  return apply (SYMB_MINUS, s1, s2);
}

symbolic
symbolic_rep::mul (const symbolic& s1, const symbolic& s2) {
  if (s1 == SYMB_ZERO) return SYMB_ZERO;
  if (s2 == SYMB_ZERO) return SYMB_ZERO;
  if (s1 == SYMB_ONE) return s2;
  if (s2 == SYMB_ONE) return s1;
  return apply (SYMB_TIMES, s1, s2);
}

symbolic
symbolic_rep::div (const symbolic& s1, const symbolic& s2) {
  if (s1 == SYMB_ZERO) return SYMB_ZERO;
  if (s2 == SYMB_ONE) return s1;
  return apply (SYMB_OVER, s1, s2);
}

symbolic
symbolic_rep::pow (const symbolic& s1, const symbolic& s2) {
  if (s1 == SYMB_ZERO) return SYMB_ZERO;
  if (s1 == SYMB_ONE) return SYMB_ONE;
  if (s2 == SYMB_ZERO) return SYMB_ONE;
  if (s2 == SYMB_ONE) return s1;
  return apply (SYMB_POWER, s1, s2);
}

symbolic
operator + (const symbolic& s1, const symbolic& s2) {
  if (s1->priority () > s2->priority ())
    return s1->add (s1, s2);
  else return s2->add (s1, s2);
}

symbolic
operator - (const symbolic& s1) {
  return s1->neg (s1);
}

symbolic
operator - (const symbolic& s1, const symbolic& s2) {
  if (s1->priority () > s2->priority ())
    return s1->sub (s1, s2);
  else return s2->sub (s1, s2);
}

symbolic
operator * (const symbolic& s1, const symbolic& s2) {
  if (s1->priority () > s2->priority ())
    return s1->mul (s1, s2);
  else return s2->mul (s1, s2);
}

symbolic
operator / (const symbolic& s1, const symbolic& s2) {
  if (s1->priority () > s2->priority ())
    return s1->div (s1, s2);
  else return s2->div (s1, s2);
}

symbolic
pow (const symbolic& s1, const symbolic& s2) {
  if (s1->priority () > s2->priority ())
    return s1->pow (s1, s2);
  else return s2->pow (s1, s2);
}

} // namespace mml
