
/******************************************************************************
* MODULE     : symbolic_compound.cpp
* DESCRIPTION: Compound symbolic expressions
* COPYRIGHT  : (C) 2004  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "symbolic.hpp"
namespace mml {
#define GET(s) ((symbolic_compound_rep*) s.rep)->data

class symbolic_compound_rep: public symbolic_rep {
public:
  vector<symbolic> data;

protected:
  // accessors and syntactic identity
  sym_type type () { return TYPE_COMPOUND; }
  bool is_atomic () { return false; }
  nat arity () { return dim (data) - 1; }
  symbolic child (nat i) { return data[i]; }
  nat hash () { return mml::hash (data); }
  bool identical (symbolic_rep* rep) {
    if (rep->type() != TYPE_COMPOUND) return false;
    return data == ((symbolic_compound_rep*) rep)->data;
  }

public:
  symbolic_compound_rep (const vector<symbolic>& data2): data (data2) {}
};

symbolic
symbolic_compound (const vector<symbolic>& v) {
  return new symbolic_compound_rep (v);
}

symbolic
apply (const symbolic& fun) {
  return symbolic_compound (vec (fun));
}

symbolic
apply (const symbolic& fun, const symbolic& s1) {
  return symbolic_compound (vec (fun, s1));
}

symbolic
apply (const symbolic& fun, const symbolic& s1, const symbolic& s2) {
  return symbolic_compound (vec (fun, s1, s2));
}

symbolic
apply (const symbolic& fun, const vector<symbolic>& v) {
  return symbolic_compound (append (vec (fun), v));
}

} // namespace mml
