
/******************************************************************************
* MODULE     : symbolic_integer.cpp
* DESCRIPTION: Symbolic integers
* COPYRIGHT  : (C) 2004  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "symbolic_aux.hpp"
#include "rational.hpp"
namespace mml {
#define GET(s) ((symbolic_integer_rep*) s.rep)->data

class symbolic_integer_rep: public symbolic_rep {
public:
  integer data;

protected:
  // accessors and syntactic identity
  sym_type type () { return TYPE_INTEGER; }
  string leaf () { return print_to_string<integer> (data); }
  nat hash () { return mml::hash (data); }
  bool identical (symbolic_rep* rep) {
    if (rep->type() != TYPE_INTEGER) return false;
    return data == ((symbolic_integer_rep*) rep)->data;
  }

  // arithmetic
  symbolic add (const symbolic& s1, const symbolic& s2) {
    if (s1->type() != s2->type()) return symbolic_rep::add (s1, s2);
    return GET (s1) + GET (s2);
  }
  symbolic neg (const symbolic& s1) {
    return -GET (s1);
  }
  symbolic sub (const symbolic& s1, const symbolic& s2) {
    if (s1->type() != s2->type()) return symbolic_rep::sub (s1, s2);
    return GET (s1) - GET (s2);
  }
  symbolic mul (const symbolic& s1, const symbolic& s2) {
    if (s1->type() != s2->type()) return symbolic_rep::mul (s1, s2);
    return GET (s1) * GET (s2);
  }
  symbolic div (const symbolic& s1, const symbolic& s2) {
    if (s1->type() != s2->type()) return symbolic_rep::div (s1, s2);
    return rational (GET (s1)) / rational (GET (s2));
  }

public:
  symbolic_integer_rep (const integer& data2): data (data2) {}
};

symbolic::symbolic () {
  rep= SYMB_ZERO.rep; INC_COUNT (rep);
}

symbolic::symbolic (int i) {
  rep= memorize (new symbolic_integer_rep (integer (i)));
}

symbolic::symbolic (const integer& i) {
  rep= memorize (new symbolic_integer_rep (i));
}

symbolic_rep*
symbolic_integer (const integer& i) {
  return new symbolic_integer_rep (i);
}

integer
as_integer (const symbolic& s) {
  assert (s->type () == TYPE_INTEGER);
  return ((symbolic_integer_rep*) s.rep)->data;
}

} // namespace mml
