
/******************************************************************************
* MODULE     : symbolic_rational.cpp
* DESCRIPTION: Symbolic rationals
* COPYRIGHT  : (C) 2004  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "symbolic_aux.hpp"
#include "rational.hpp"
namespace mml {
#define RATIONAL(s) ((s)->type()==TYPE_INTEGER || (s)->type()==TYPE_RATIONAL)
#define GET(s) as_rational(s)
symbolic_rep* symbolic_integer (const integer& i);

class symbolic_rational_rep: public symbolic_rep {
public:
  rational data;

protected:
  // accessors and syntactic identity
  sym_type type () { return TYPE_RATIONAL; }
  bool is_atomic () { return false; }
  nat arity () { return 2; }
  symbolic child (nat i) {
    if (i==0) return "/";
    else if (i==1) return num (data);
    else return den (data); }
  nat hash () { return mml::hash (data); }
  bool identical (symbolic_rep* rep) {
    if (rep->type() != TYPE_RATIONAL) return false;
    return data == ((symbolic_rational_rep*) rep)->data;
  }

  // arithmetic
  symbolic add (const symbolic& s1, const symbolic& s2) {
    if (RATIONAL (s1) && RATIONAL (s2))
      return GET (s1) + GET (s2);
    else return symbolic_rep::add (s1, s2);
  }
  symbolic neg (const symbolic& s1) {
    return -GET (s1);
  }
  symbolic sub (const symbolic& s1, const symbolic& s2) {
    if (RATIONAL (s1) && RATIONAL (s2))
      return GET (s1) - GET (s2);
    else return symbolic_rep::sub (s1, s2);
  }
  symbolic mul (const symbolic& s1, const symbolic& s2) {
    if (RATIONAL (s1) && RATIONAL (s2))
      return GET (s1) * GET (s2);
    else return symbolic_rep::mul (s1, s2);
  }
  symbolic div (const symbolic& s1, const symbolic& s2) {
    if (RATIONAL (s1) && RATIONAL (s2))
      return GET (s1) / GET (s2);
    else return symbolic_rep::div (s1, s2);
  }

public:
  symbolic_rational_rep (const rational& data2): data (data2) {}
};

symbolic::symbolic (const rational& r) {
  if (den (r) == 1) rep= memorize (symbolic_integer (num (r)));
  else rep= memorize (new symbolic_rational_rep (r));
}

rational
as_rational (const symbolic& s) {
  if (s->type () == TYPE_INTEGER)
    return rational (as_integer (s));
  assert (s->type () == TYPE_RATIONAL);
  return ((symbolic_rational_rep*) s.rep)->data;
}

} // namespace mml
